-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local im  = ui_imgui

local C = {}

C.name = 'Restart AI Race Vehicle'
C.description = 'Restart AI Race Vehicle for extra AI race mod.'
C.category = 'repeat_instant'
C.color = im.ImVec4(0.4, 0.9, 1.0, 0.9)
C.author = 'mathkuro'
C.tmpSecondPassFlag = true
C.pinSchema = {
  {dir = 'in', type = 'number', name = 'vehId', description = 'Vehicle id.'},
  {dir = 'in', type = 'table', name = 'curAIPath', description = 'Current AI race path.'},
  {dir = 'in', type = 'number', name = 'curCP', description = 'Current check point.'},
  {dir = 'in', type = 'number', name = 'maxCP', description = 'Max check point.'},
  {dir = 'in', type = 'number', name = 'curLap', description = 'Current lap.'},
  {dir = 'in', type = 'number', name = 'maxLap', description = 'Max lap.'},
  {dir = 'in', type = 'number', name = 'racerAggression', description = 'Current racer aggression.'}
}

C.tags = {}

function C:init()
  self.speedThreshold = 3
  self.damageThreshold = 2000
  self.stopFrameThreshold = 300
  self.stopFrameCount = 0
  self.isRecovered = false
  self.aiRestartDelayThreshold = 60
  self.aiRestartDelayCount = 0
end

function C:work()
  -- If race finished
  -- print('curLap: ' .. tostring(self.pinIn.curLap.value) .. ', maxLap: ' .. tostring(self.pinIn.maxLap.value) .. ', curCP: ' .. tostring(self.pinIn.curCP.value) .. ', maxCP: ' .. tostring(self.pinIn.maxCP.value))
  local daylightValues = {0.22, 0.78} -- sunset & sunrise

  local oldAIPath = self.pinIn.curAIPath.value
  local lapCount = self.pinIn.maxLap.value + 1
  local aiPath = {}
  -- game speed( before unit convertion)
  local _vehData = map.objects[self.pinIn.vehId.value]
  local _curSpeed = _vehData.vel:length()
  local _curDamage = _vehData.damage
  -- print('_curSpeed: ' .. tostring(_curSpeed) .. ', _curDamage: ' .. tostring(_curDamage))

  if self.isRecovered then
    self.aiRestartDelayCount = self.aiRestartDelayCount + 1
    if self.aiRestartDelayThreshold < self.aiRestartDelayCount then
        self.aiRestartDelayCount = 0
        self.isRecovered = false
        self.stopFrameCount = 0

        local veh = be:getObjectByID(self.pinIn.vehId.value)

        veh:queueLuaCommand('electrics.set_lightbar_signal(1)')

        local currentTODTime = core_environment.getTimeOfDay()['time']
        print(currentTODTime)
        if daylightValues[1] < currentTODTime and currentTODTime < daylightValues[2] then
            veh:queueLuaCommand('electrics.setLightsState(2)')
            print('electrics.setLightsState(2)')
        end

        if not self.pinIn.curCP.value then
          aiPath = oldAIPath
        else
          print('curLap: ' .. tostring(self.pinIn.curLap.value) .. ', maxLap: ' .. tostring(self.pinIn.maxLap.value) .. ', curCP: ' .. tostring(self.pinIn.curCP.value) .. ', maxCP: ' .. tostring(self.pinIn.maxCP.value))
          print('oldAIPath: ' .. serialize(oldAIPath))
          for k, v in pairs(oldAIPath) do
              if k >= self.pinIn.curCP.value then
                  table.insert(aiPath, v)
              end
          end
          for k, v in pairs(oldAIPath) do
              if k < self.pinIn.curCP.value then
                  table.insert(aiPath, v)
              end
          end
          if lapCount >= 1 then
            -- If the path is closed
            table.insert(aiPath, oldAIPath[self.pinIn.curCP.value])
          end
        end

        local cmd = "ai.driveUsingPath({wpTargetList = " .. serialize(aiPath) .. ", noOfLaps = " .. tostring(lapCount) .. ", aggression = " .. tostring(self.pinIn.racerAggression.value)  .. ", avoidCars = 'on'})"
        veh:queueLuaCommand(cmd)
        print(cmd)
        return
    end
    return
  end

  -- If the racer does not reach first checkpoint.
  if self.pinIn.curLap.value >= self.pinIn.maxLap.value and not self.pinIn.curCP.value then
    return
  end

  -- If the speed is below threshold reset certain speed 
  if self.speedThreshold < _curSpeed then
    self.stopFrameCount = 0
    return
  end
  self.stopFrameCount = self.stopFrameCount + 1

  -- If the rival vehicle stop long time, recover it to last check point
  if self.stopFrameCount < self.stopFrameThreshold then
    return
  end
  print('Rival Racer: ' .. tostring(self.pinIn.vehId.value) .. ' is recovered.')
  self.isRecovered = true
  local veh = be:getObjectByID(self.pinIn.vehId.value)
  veh:queueLuaCommand('recovery.loadHome()')

end

return _flowgraph_createNode(C)
